@@ -8,12 +8,13 @@ import regex
 import re
 from decimal import Decimal
 
-
 from npf.node import Node
 from .variable import VariableFactory
 
+import numpy as np
 
 options = None
+cwd = None
 
 def get_valid_filename(s):
     s = str(s).strip().replace(' ', '_')
@@ -29,7 +30,7 @@ class ExtendAction(argparse.Action):
 
 def add_verbosity_options(parser: ArgumentParser):
     v = parser.add_argument_group('Verbosity options')
-    v.add_argument('--show-full', help='Show full execution results',
+    v.add_argument('--show-full', '--show-all', help='Show full execution results',
                    dest='show_full', action='store_true',
                    default=False)
     v.add_argument('--show-files', help='Show content of created files',
@@ -56,10 +57,11 @@ def add_verbosity_options(parser: ArgumentParser):
 def add_graph_options(parser: ArgumentParser):
     o = parser.add_argument_group('Output data')
     o.add_argument('--output',
-                   help='Output data to CSV', dest='output', type=str, nargs='?', const='graph', default=None)
-    o.add_argument('--pandas', metavar='pandas_filename', type=str, default=None, dest='pandas_filename',
-                    help='Output a Pandas dataframe to CSV')
-    o.add_argument('--output-columns', dest='output_columns', type=str, nargs='+', default=['x', 'mean'])
+                   help='Output data to CSV files, one per result type. By default it prints the variable value as first column and the second column is the mean of all runs. Check --output-columns to change this behavior.', dest='output', type=str, nargs='?', const='graph', default=None)
+    o.add_argument('--output-columns', dest='output_columns', type=str, nargs='+', default=['x', 'mean'],
+                    help='Columns to print in each --output CSV files. By default x mean. Valid values are mean/average, min, max, perc[0-9]+, med/median/perc50, std, nres/n, first, last, all. Check the documentation for details.')
+    o.add_argument('--single-output', '--pandas', metavar='pandas_filename', type=str, default=None, dest='pandas_filename',
+                    help='Output a dataframe to CSV with all result types as columns, and one line per variables and per runs. This is a pandas dataframe, and can be read with pandas easily again. Time series are each outputed in a single different CSV.')
 
     g = parser.add_argument_group('Graph options')
     g.add_argument('--graph-size', metavar='INCH', type=float, nargs=2, default=None,
@@ -72,8 +74,9 @@ def add_graph_options(parser: ArgumentParser):
 
     g.add_argument('--graph-no-series', dest='graph_no_series', action='store_true', default=False)
 
-    g.add_argument('--graph-group-repo', dest='group_repo', action='store_true', default=False)
+    g.add_argument('--graph-group-repo', dest='group_repo', action='store_true', default=False, help="Group series, using the repository name as a variable")
 
+    g.add_argument('--graph-group-series', dest='group_series', action='store_true', default=False, help="Group series with the same name")
     g.add_argument('--no-transform', dest='do_transform', action='store_false', default=True, help="Forbid automatic transformation of data such as extracting a variable as a serie")
 
     g.add_argument('--graph-select-max', dest='graph_select_max', type=int, default=None)
@@ -146,17 +149,26 @@ def add_testing_options(parser: ArgumentParser, regression: bool = False):
     t.add_argument('--testie', '--test', '--npf', dest='test_files', metavar='path or testie', type=str, nargs='?', default='tests',
                    help='script or script folder. Default is tests')
 
-    t.add_argument('--cluster', metavar='user@address:path', type=str, nargs='*', default=[],
-                   help='role to node mapping for remote execution of tests')
 
     t.add_argument('--build-folder', '--build-path', metavar='path', type=str, default=None, dest='build_folder', help='Set where dependencies would be built. Defaults to npf\'s folder itself, so dependencies are shared between test scripts.')
 
+    t.add_argument('--experiment-folder', '--experiment-path', metavar='path', type=str, default="./", dest='experiment_folder', help='Where to create temporary files and execute tests from. Default to local directory. Tests will always be executed from a temporary-made folder inside that path. Beware if you do not have a similar NFS/SSHFS based directory identical on all nodes, the "path" argument of the cluster file must match this repository, same as the build path relative to that directory.')
+
     t.add_argument('--search-path', metavar='path', type=str, default=[], nargs='+', dest='search_path', help='Search for various files in this directories too (such as the parent of your own repo, cluster or modules folder)')
     t.add_argument('--no-mp', dest='allow_mp', action='store_false',
                    default=True, help='Run tests in the same thread. If there is multiple script, they will run '
                                       'one after the other, hence breaking most of the tests.')
     t.add_argument('--expand', type=str, default=None, dest="expand")
     t.add_argument('--rand-env', type=int, default=65536, dest="rand_env")
+    t.add_argument('--experimental-design', type=str, default="matrix.csv", help="The path towards the experimental design point selection file")
+
+    c = parser.add_argument_group('Cluster options')
+    c.add_argument('--cluster', metavar='role=user@address:path [...]', type=str, nargs='*', default=[],
+                   help='role to node mapping for remote execution of tests. The format is role=address, where address can be an address or a file in cluster/address.node describing supplementary parameters for the node.')
+    c.add_argument('--cluster-autosave', default=False, action='store_true', dest='cluster_autosave',
+                    help='Automatically save NICs found on the machine. If the file cluster/address.node does not exists, NPF will attempt to auto-discover NICs. If this option is set, it will auto-create the file.')
+
+
     return t
 
 
@@ -188,7 +200,7 @@ def add_building_options(parser):
     return b
 
 nodePattern = regex.compile(
-    "(?P<role>[a-zA-Z0-9]+)=(:?(?P<user>[a-zA-Z0-9]+)@)?(?P<addr>[a-zA-Z0-9.-]+)(:?[:](?P<path>path))?")
+    "(?P<role>[a-zA-Z0-9]+)=(:?(?P<user>[a-zA-Z0-9]+)@)?(?P<addr>[a-zA-Z0-9.-]+)(:?[:](?P<path>[a-zA-Z0-9_./~-]+))?")
 roles = {}
 
 
@@ -217,38 +229,77 @@ def executor(role, default_role_map):
     return nodes_for_role(role, default_role_map)[0].executor
 
 
-def parse_nodes(args):
+def set_args(args):
     sys.modules[__name__].options = args
+    sys.modules[__name__].cwd = os.getcwd()
+
+def parse_nodes(args):
+    set_args(args)
+
     if type(options.use_last) is not int:
         if options.use_last:
             options.use_last = 100
 
-    roles['default'] = [Node.makeLocal(options)]
+    if not os.path.exists(experiment_path()):
+        raise Exception("The experiment root '%s' is not accessible ! Please explicitely define it with --experiment-path, and ensure that directory is writable !" % experiment_path())
+
+    if not os.path.isabs(options.experiment_folder):
+        options.experiment_folder = os.path.abspath(options.experiment_folder)
+
+    # Create the test file
+    os.close(os.open(experiment_path() + ".access_test" , os.O_CREAT))
+    local = Node.makeLocal(options)
+    #Delete the test file
+    os.unlink(experiment_path() + ".access_test")
+
+    roles['default'] = [local]
 
     options.search_path = set(options.search_path)
     for t in [options.test_files]:
         options.search_path.add(os.path.dirname(t))
 
     for val in options.cluster:
-        variables = val.split(',')
+
+        # Create the test file
+        os.close(os.open(experiment_path() + ".access_test" , os.O_CREAT))
+
+        variables : list[str] = val.split(',')
         if len(variables) == 0:
             raise Exception("Bad definition of cluster parameter : %s" % variables)
-        mapping=variables[0].strip()
+        mapping: str =variables[0].strip()
         match = nodePattern.match(mapping)
         if not match:
             raise Exception("Bad definition of node : %s" % mapping)
+
+        path = match.group('path')
+
+        del variables[0]
+
+        nfs = None
+        assert isinstance(variables, list)
+        for opts in variables:
+            assert isinstance(opts, str)
+            var,val = opts.split('=')
+            if var == "nfs":
+                nfs = int(val)
+            elif var == "path":
+                path = val
+            else:
+                continue
+            variables.remove(opts)
+
         if match.group('addr') == 'localhost':
-            node = roles['default'][0]
+            node = local
         else:
-            node = Node.makeSSH(user=match.group('user'), addr=match.group('addr'), path=match.group('path'),
-                            options=options)
+            node = Node.makeSSH(user=match.group('user'), addr=match.group('addr'), path=path,
+                            options=options, nfs=nfs)
         role = match.group('role')
         if role in roles:
             roles[role].append(node)
             print("Role %s has multiple nodes. The role will be executed by multiple machines. If this is not intended, fix your --cluster option." % role)
         else:
             roles[role] = [node]
-        del variables[0]
+
         for opts in variables:
             var,val = opts.split('=')
             if var == 'nic':
@@ -260,6 +311,10 @@ def parse_nodes(args):
             else:
                 raise Exception("Unknown cluster variable : %s" % var)
 
+        #Delete the test file if it still exists (if the remote is the local machine, it won't)
+        if os.path.exists(experiment_path() + ".access_test"):
+            os.unlink(experiment_path() + ".access_test")
+
 
 def parse_variables(args_variables, tags, sec) -> Dict:
     variables = {}
@@ -280,22 +335,47 @@ def override(args, testies):
     return testies
 
 
-def npf_root():
+def npf_root_path():
     # Return the path to NPF root
     return os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
 
-def from_root(path):
+def npf_writeable_root_path():
+    path = npf_root_path()
+    if not os.access(path, os.W_OK):
+        return experiment_path()
+    else:
+        return path
+
+def experiment_path():
+    # Return the path to NPF experiment folder
+    options = sys.modules[__name__].options
+    return os.path.join(options.experiment_folder,'')
+
+def cwd_path():
+    # Return the path to where NPF was first executed
+    return sys.modules[__name__].cwd
+
+def get_build_path():
+    options = sys.modules[__name__].options
+    if not options.build_folder is None:
+        if not os.access(options.build_folder, os.W_OK):
+            raise Exception("The provided build path is not writeable or does not exists : %s!" % options.build_folder)
+        return options.build_folder
+    else:
+        return npf_writeable_root_path()+'/build/'
+
+def from_experiment_path(path):
     # Returns the path under NPF root if it is not absolute
     if (os.path.isabs(path)):
         return path
     else:
-        return npf_root() + os.sep + path
+        return (experiment_path() if os.path.isabs(experiment_path()) else os.path.abspath(experiment_path())) + os.sep + path
 
-def find_local(path, critical: bool = False):
+def find_local(path, critical: bool = False, suppl: List = None):
     if os.path.exists(path):
         return path
 
-    searched = [npf_root(), '.'] + list(sys.modules[__name__].options.search_path)
+    searched = [npf_root_path(), '.', experiment_path()] + list(sys.modules[__name__].options.search_path) + (suppl if suppl else [])
     for a in searched:
         p = a + os.sep + path
         if os.path.exists(p):
@@ -326,9 +406,9 @@ def build_filename(testie, build, hint, variables, def_ext, type_str='', show_se
 
     if hint is None:
         if data_folder:
-            path = build.result_path(testie.filename, def_ext, folder = var_str + (('-' if var_str else '') + type_str if type_str else ''), prefix=prefix, suffix = ('-' + get_valid_filename(build.pretty_name()) if show_serie else ''))
+            path = build.result_path(testie.filename, def_ext, folder = var_str + (('-' if var_str else '') + type_str if type_str else ''), prefix=prefix, suffix = ('-' + suffix if suffix else '') + ('-' + get_valid_filename(build.pretty_name()) if show_serie else ''))
         else:
-            path = build.result_path(testie.filename, def_ext, suffix=var_str + ('-' + type_str if type_str else '') + ('-' + get_valid_filename(build.pretty_name()) if show_serie else '') , prefix=prefix)
+            path = build.result_path(testie.filename, def_ext, suffix=('-' + suffix if suffix else '') + var_str + ('-' + type_str if type_str else '') + ('-' + get_valid_filename(build.pretty_name()) if show_serie else '') , prefix=prefix)
     else:
         dirname, basename, ext = splitpath(hint)
 
@@ -380,7 +460,7 @@ def parseBool(s):
        return bool(s)
 
 def parseUnit(u):
-    r = re.match('([-]?)([0-9]+)[ ]*([GMK]?)',u)
+    r = re.match('([-]?)([0-9.]+)[ ]*([GMK]?)',u)
     if r != None:
         n = float(r.group(2))
         unit = r.group(3)
@@ -403,6 +483,6 @@ def parseUnit(u):
 
 def all_num(l):
     for x in l:
-        if type(x) is not int and type(x) is not float and type(x) is not Decimal:
+        if type(x) is not int and type(x) is not Decimal and not isinstance(x, (np.floating, float)):
             return False
     return True
