@@ -40,7 +40,8 @@ for sect in known_sections:
 class SectionFactory:
     varPattern = "([a-zA-Z0-9_:-]+)[=](" + Variable.VALUE_REGEX + ")?"
     namePattern = re.compile(
-        "^(?P<tags>" + Variable.TAGS_REGEX + "[:])?(?P<name>info|config|variables|exit|pypost|pyexit|late_variables|include (?P<includeName>[a-zA-Z0-9_./-]+)||(init-)?file(:?[@](?P<fileRole>[a-zA-Z0-9]+))? (?P<fileName>[a-zA-Z0-9_.-]+)(:? (?P<fileNoparse>noparse))?|require|"
+        "^(?P<tags>" + Variable.TAGS_REGEX + "[:])?(?P<name>info|config|variables|exit|pypost|pyexit|late_variables|include (?P<includeName>[a-zA-Z0-9_./-]+)(?P<includeParams>([ \t]+" +
+        varPattern + ")+)?|(init-)?file(:?[@](?P<fileRole>[a-zA-Z0-9]+))? (?P<fileName>[a-zA-Z0-9_.-]+)(:? (?P<fileNoparse>noparse))?|require|"
                                              "import(:?[@](?P<importRole>[a-zA-Z0-9]+)(:?[-](?P<importMulti>[*0-9]+))?)?[ \t]+(?P<importModule>" + Variable.VALUE_REGEX + ")(?P<importParams>([ \t]+" +
         varPattern + ")+)?|" +
                      "sendfile(:?[@](?P<sendfileRole>[a-zA-Z0-9]+))?[ \t]+(?P<sendfilePath>.*)|" +
@@ -105,7 +106,9 @@ class SectionFactory:
                                 noparse=matcher.group('fileNoparse'))
             return s
         elif sectionName.startswith('include'):
-            s = SectionImport(None, matcher.group('includeName').strip(), {}, is_include=True)
+            params = matcher.group('includeParams')
+            params = dict(re.findall(SectionFactory.varPattern, params)) if params else {}
+            s = SectionImport(None, matcher.group('includeName').strip(), params, is_include=True)
             return s
         elif sectionName == 'require':
             s = SectionRequire()
@@ -289,15 +292,20 @@ class BruteVariableExpander:
     """Expand all variables building the full
     matrix first."""
 
-    def __init__(self, vlist):
+    def __init__(self, vlist, overriden):
         self.expanded = [OrderedDict()]
         for k, v in vlist.items():
+            if k in overriden:
+                continue
             newList = []
-            for nvalue in v.makeValues():
+            l = v.makeValues()
+
+            for nvalue in l:
                 for ovalue in self.expanded:
                     z = ovalue.copy()
-                    z.update({k: nvalue})
+                    z.update(nvalue if type(nvalue) is OrderedDict else {k: nvalue})
                     newList.append(z)
+
             self.expanded = newList
         self.it = self.expanded.__iter__()
 
@@ -335,14 +343,14 @@ class SectionVariable(Section):
             values.append(SectionVariable.replace_variables(v, value))
         return values
 
-    def expand(self, method=None):
+    def expand(self, method=None, overriden=set()):
         if method == "shuffle" or method == "rand" or method == "random":
             return RandomVariableExpander(self.vlist)
         else:
-            return BruteVariableExpander(self.vlist)
+            return BruteVariableExpander(self.vlist, overriden)
 
     def __iter__(self):
-        return BruteVariableExpander(self.vlist)
+        return self.expand()
 
     def __len__(self):
         if len(self.vlist) == 0:
@@ -376,20 +384,53 @@ class SectionVariable(Section):
         for k, v in d.items():
             self.override(k, v)
 
+    @staticmethod
+    def _assign(vlist, assign, var, val):
+                    cov = None
+                    for k,v in vlist.items():
+                        if isinstance(v,CoVariable):
+                            if var in v.vlist:
+                                cov = v.vlist
+                                break
+
+                    if assign == '+=':
+                        #If in covariable, remove that one
+                        if cov:
+                            print("NOTE: %s is overwriting a covariable, the covariable will be ignored" % var)
+                            del cov[var]
+                        if var in vlist:
+                            vlist[var] += val
+                        else:
+                            vlist[var] = val
+                    elif assign == '?=':
+                        #If in covariable or already in vlist, do nothing
+                        if not cov and not var in vlist:
+                            vlist[var] = val
+                    else:
+                        #If in covariable, remove it as we overwrite the value
+                        if cov:
+                            del cov[var]
+                        vlist[var] = val
     def override(self, var, val):
-        if not var in self.vlist:
+        found = False
+        for k,v in self.vlist.items():
+            if isinstance(v, CoVariable):
+                if var in v.vlist:
+                    found = True
+                    break
+            if k == var:
+                found = True
+                break
+        if not found:
             print("WARNING : %s does not override anything" % var)
-        if isinstance(val, Variable):
+
+        if not isinstance(val, Variable):
+            val = SimpleVariable(var, val)
+        else:
             if val.is_default:
                 return
-            if val.assign == '+=':
-                self.vlist[var] += val
-            elif val.assign == '?=' and not var in self.vlist:
-                self.vlist[var] = val
-            else:
-                self.vlist[var] = val
-        else:
-            self.vlist[var] = SimpleVariable(var, val)
+
+        self._assign(self.vlist, val.assign, var, val)
 
     @staticmethod
     def match_tags(text, tags):
@@ -435,29 +476,36 @@ class SectionVariable(Section):
             else:
                 return None, None, False
 
-    def build(self, content, testie, check_exists=False, fail=True):
+    def build(self, content:str, testie, check_exists:bool=False, fail:bool=True):
+        sections_stack = [self]
         for line in content.split("\n"):
-            var, val, assign = self.parse_variable(line, testie.tags, vsection=self, fail=fail)
-            if not var is None and not val is None:
-                if check_exists and not var in self.vlist:
-
-                    if var.endswith('s') and var[:-1] in self.vlist:
-                        var = var[:-1]
-                    elif var + 's' in self.vlist:
-                        var = var + 's'
-                    else:
-                        if var in self.aliases:
-                            var = self.aliases[var]
+            if line.strip() == "{":
+                c = CoVariable()
+                sections_stack.append(c)
+                self.vlist[c.name] = c
+            elif line.strip() == "}":
+                sections_stack.pop()
+                for k in c.vlist.keys():
+                    if k in self.vlist:
+                        del self.vlist[k]
+            else:
+                line = line.lstrip()
+                sect = sections_stack[-1]
+                var, val, assign = self.parse_variable(line, testie.tags, vsection=sect, fail=fail)
+                if not var is None and not val is None:
+                    # If check_exists, we verify that we overwrite a variable. This is used by config section to ensure we write known parameters
+                    if check_exists and not var in sect.vlist:
+                        if var.endswith('s') and var[:-1] in sect.vlist:
+                            var = var[:-1]
+                        elif var + 's' in sect.vlist:
+                            var = var + 's'
                         else:
-                            raise Exception("Unknown variable %s" % var)
-                if assign == '+=' and var in self.vlist:
-                    self.vlist[var] += val
-                elif assign == '?=':
-                    if not var in self.vlist:
-                        self.vlist[var] = val
-                else:
-                    self.vlist[var] = val
-        return OrderedDict(sorted(self.vlist.items()))
+                            if var in self.aliases:
+                                var = self.aliases[var]
+                            else:
+                                raise Exception("Unknown variable %s" % var)
+                    self._assign(sect.vlist, assign, var, val)
+        return OrderedDict(self.vlist.items())
 
     def finish(self, testie):
         self.vlist = self.build(self.content, testie)
@@ -466,10 +514,14 @@ class SectionVariable(Section):
         formats = []
         names = []
         for k, v in self.vlist.items():
-            f = v.format()
-            formats.append(f)
-            names.append(k)
-        return dict(names=names, formats=formats)
+            k, f = v.format()
+            if type(f) is list:
+                formats.extend(f)
+                names.extend(k)
+            else:
+                formats.append(f)
+                names.append(k)
+        return OrderedDict(names=names, formats=formats)
 
 
 class SectionLateVariable(SectionVariable):
@@ -518,7 +570,11 @@ class SectionConfig(SectionVariable):
 
         self.aliases = {
             'graph_variable_as_series': 'graph_variables_as_series',
+            'graph_variable_as_serie': 'graph_variables_as_series',
+            'graph_variables_as_serie': 'graph_variables_as_series',
+            'graph_subplot_variables' : 'graph_subplot_variable',
             'graph_grid': 'var_grid',
+            'graph_ticks' : 'var_ticks',
             'graph_serie': 'var_serie',
             'graph_types':'graph_type',
             'graph_linestyle': 'graph_lines',
@@ -529,14 +585,18 @@ class SectionConfig(SectionVariable):
             'y_group': 'graph_y_group',
             'graph_serie_sort':'graph_series_sort',
             'series_prop': 'graph_series_prop',
-            'graph_legend_ncol': 'legend_ncol'
+            'graph_legend_ncol': 'legend_ncol',
+            'graph_legend_loc': 'legend_loc',
+            'var_label_dir' : 'graph_label_dir',
+            'graph_max_col' : 'graph_max_col',
+
         }
 
         # Environment
-        self.__add("default_repo", None)
+        self.__add("default_repo", "local")
 
         # Regression related
-        self.__add_list("accept_zero", ["time"])
+        self.__add_list("accept_zero", ["time","DROP", "DROPPED"])
         self.__add("n_supplementary_runs", 3)
         self.__add("acceptable", 0.01)
         self.__add("accept_outliers_mult", 1)
@@ -576,16 +636,18 @@ class SectionConfig(SectionVariable):
         self.__add("graph_subplot_unique_legend", False)
         self.__add_list("graph_display_statics", [])
         self.__add_list("graph_variables_as_series", [])
+        self.__add("graph_variables_explicit", False)
         self.__add_list("graph_hide_variables", [])
         self.__add_dict('graph_result_as_variable', {})
         self.__add_dict('graph_map', {})
         self.__add_dict('graph_x_sort', {})
         self.__add("graph_scatter", False)
         self.__add("graph_show_values", False)
-
+        self.__add("graph_show_ylabel", True)
         self.__add("graph_show_xlabel", True)
         self.__add("graph_subplot_type", "subplot")
         self.__add("graph_max_series", None)
+        self.__add("graph_max_cols", 2)
         self.__add("graph_series_as_variables", False)
         self.__add("graph_series_prop", False)
         self.__add("graph_series_sort", None)
@@ -614,14 +676,14 @@ class SectionConfig(SectionVariable):
         self.__add_dict("var_ticks", {})
 
         self.__add_dict("graph_legend_params", {})
-        self.__add_list("var_grid", [])
-        self.__add("graph_grid_linestyle", "-")
+        self.__add_list("var_grid", ["result"])
+        self.__add("graph_grid_linestyle", ":")
 
         self.__add("graph_fillstyle", "full")
         self.__add_dict("graph_tick_params", {})
         self.__add_list("test_time_sync", [])
         self.__add("var_serie",None)
-        self.__add_dict("var_names", {"result-LATENCY":"Latency (µs)","result-THROUGHPUT":"Throughput","boxplot":""})
+        self.__add_dict("var_names", {"result-LATENCY":"Latency (µs)", "result-THROUGHPUT":"Throughput", "^THROUGHPUT$":"Throughput", "boxplot":"", "^PARALLEL$":"Number of parallel connections", "^ZEROCOPY$":"Zero-Copy"})
         self.__add_dict("var_unit", {"result": "","result-LATENCY":"us","latency":"us","throughput":"bps"})
         self.__add("graph_show_fliers", True)
         self.__add_dict("graph_cross_reference", {})
@@ -633,7 +695,7 @@ class SectionConfig(SectionVariable):
         self.__add_list("graph_type", [])
         self.__add("title", None)
         self.__add_list("require_tags", [])
-        self.__add_dict("var_label_dir", {})
+        self.__add_dict("graph_label_dir", {})
         self.__add("graph_force_diagonal_labels", False)
         self.__add("graph_smooth", 1)
 
@@ -728,9 +790,12 @@ class SectionConfig(SectionVariable):
         self.__add(key.lower(), val)
 
     def match(self, key, val):
-        for match in self.get_list(key):
-            if re.match(match, val):
-                return True
+        try:
+            for match in self.get_list(key):
+                if re.match(match, val):
+                    return True
+        except sre_constants.error:
+            print("ERROR : Regex %s does not work" % key)
         return False
 
     def finish(self, testie):
