@@ -21,6 +21,7 @@ from npf.types.dataset import Run, Dataset
 from npf.eventbus import EventBus
 from .variable import get_bool
 from decimal import *
+from functools import reduce
 
 from subprocess import PIPE, Popen, TimeoutExpired
 
@@ -52,14 +53,16 @@ class RemoteParameters:
 def _parallel_exec(param: RemoteParameters):
     nodes = npf.nodes_for_role(param.role)
     executor = nodes[param.role_id].executor
-    if param.waitfor:
+    for wf in param.waitfor if type(param.waitfor) is list else [param.waitfor]:
+        if wf is None:
+            continue
         n=1
-        #print("Waiting for %s" % param.waitfor)
-        if param.waitfor[0].isdigit():
-            n=int(param.waitfor[0])
-            param.waitfor=param.waitfor[1:]
+        #print("Waiting for %s" % wf)
+        if wf[0].isdigit():
+            n=int(wf[0])
+            wf=wf[1:]
         for i in range(n):
-            param.event.listen(param.waitfor)
+            param.event.listen(wf)
 
     param.event.wait_for_termination(param.delay)
     if param.event.is_terminated():
@@ -99,13 +102,15 @@ class ScriptInitException(Exception):
 
 
 class Testie:
+    __test__ = False
+
     def get_name(self):
         return self.filename
 
     def get_scripts(self) -> List[SectionScript]:
         return self.scripts
 
-    def __init__(self, testie_path, options, tags=None, role=None):
+    def __init__(self, testie_path, options, tags=None, role=None, inline=None):
         loc_path = npf.find_local(testie_path)
         if os.path.exists(loc_path):
             testie_path = loc_path
@@ -113,7 +118,7 @@ class Testie:
             loc_path = npf.find_local(testie_path + '.npf')
             if not os.path.exists(loc_path):
                 if os.path.exists(npf.find_local(testie_path + '.testie')):
-                    printf("WARNING: .testie extension is deprecated, use .npf")
+                    print("WARNING: .testie extension is deprecated, use .npf")
                     testie_path = npf.find_local(testie_path + '.testie')
                 else:
                     raise FileNotFoundError("Could not find test script %s (tried also with .npf and .testie extensions, without success)" % testie_path)
@@ -129,7 +134,7 @@ class Testie:
         self.requirements = []
         self.sendfile = {}
         self.filename = os.path.basename(testie_path)
-        self.path = os.path.dirname(testie_path)
+        self.path = os.path.dirname(os.path.abspath(testie_path))
         self.options = options
         self.tags = tags if tags else []
         self.role = role
@@ -148,6 +153,10 @@ class Testie:
                     continue
 
                 if line.startswith("%"):
+                    #Allow to start a line with a % using %% to indicate it's not a section
+                    if line[1] == '%':
+                        section.content += line[1:]
+                        continue
                     result = line[1:]
                     section = SectionFactory.build(self, result.strip())
 
@@ -196,7 +205,7 @@ class Testie:
         # Create imports testies
         for imp in self.imports:
             from npf.module import Module
-            imp.testie = Module(imp.module, options, self, imp, tags, imp.get_role())
+            imp.testie = Module(imp.module, options, self, imp, self.tags, imp.get_role())
             if len(imp.testie.variables.dynamics()) > 0:
                 raise Exception("Imports cannot have dynamic variables. Their parents decides what's dynamic.")
             if 'as_init' in imp.params:
@@ -205,14 +214,23 @@ class Testie:
             if 'delay' in imp.params:
                 delay = imp.params.setdefault('delay', 0)
                 for script in imp.testie.scripts:
-                    if not 'delay' in script.params:
-                        script.params['delay'] = float(delay) + float(imp.params['delay'])
+                    if 'delay' in script.params:
+                        script.params['delay'] = float(delay) + float(script.params['delay'])
+                    else:
+                        script.params['delay'] = float(delay)
                 del imp.params['delay']
             if 'waitfor' in imp.params:
                 for script in imp.testie.scripts:
+                    wf = imp.params['waitfor']
+                    if type(wf) is not list:
+                        wf=[wf]
                     if script.type == SectionScript.TYPE_SCRIPT:
                         if not 'waitfor' in script.params:
-                            script.params['waitfor'] = imp.params['waitfor']
+                            script.params['waitfor'] = wf
+                        else:
+                            if type(script.params['waitfor']) is not list:
+                                script.params['waitfor'] = [script.params['waitfor']]
+                            script.params['waitfor'].expand(wf)
                 del imp.params['waitfor']
             if 'autokill' in imp.params:
                 for script in imp.testie.scripts:
@@ -227,9 +245,15 @@ class Testie:
             for k, v in imp.params.items():
                 overriden_variables[k] = VariableFactory.build(k, v)
             imp.testie.variables.override_all(overriden_variables)
-
             if "require_tags" in imp.testie.config.vlist:
-                imp.testie.config.vlist["require_tags"] = imp.testie.config.vlist["require_tags"].pop("import")
+                tags = imp.testie.config.vlist["require_tags"].makeValues()
+                if tags:
+                    tags = reduce(list.__add__,map(lambda x:x.split(','), tags))
+
+                if "import" in tags:
+                    tags.remove("import")
+
+                imp.testie.config.vlist["require_tags"] = ListVariable(imp.testie.config.vlist["require_tags"].name, tags)
 
             imp.testie.config.override_all(self.config.vlist)
             self.config = imp.testie.config
@@ -277,19 +301,32 @@ class Testie:
             for node in nodes:
               if not node.nfs:
                 for repo in repo_under_test:
-                    if repo.get_reponame() != 'local' and not no_build:
-                        toSend.add((repo.reponame,role,node, repo.get_build_path()))
+                    if repo.get_build_path() and not no_build:
+                        toSend.add((repo.reponame,role,node, repo.get_build_path(), repo.get_remote_build_path(node)))
                 for dep in script.get_deps():
                     deprepo = Repository.get_instance(dep, self.options)
 
-                    toSend.add((deprepo.reponame,role,node,deprepo.get_build_path()))
-        for repo,role,node,bp in toSend.difference(done):
+                    toSend.add((deprepo.reponame,role,node,deprepo.get_build_path(), deprepo.get_remote_build_path(node)))
+        for repo,role,node,bp,rbp in toSend.difference(done):
             print("Sending software %s to %s... " % (repo, role), end ='')
-            t = node.executor.sendFolder(os.path.relpath(bp,npf.npf_root()),local=npf.npf_root())
-            if (t > 0):
-                print("%d bytes sent." % t)
+            try:
+
+                #We have to find the local path from which the remote start, so we can advance in the folder at the same point
+                local = os.path.normpath(bp)
+                r = os.path.normpath(rbp)
+                while os.path.basename(local) == os.path.basename(r):
+                    local = os.path.dirname(os.path.normpath(local))
+                    r = os.path.dirname(os.path.normpath(r))
+                t,s = node.executor.sendFolder(rbp,local)
+            except Exception as e:
+                print ("While sending %s (to folder %s) on node %s= " %  (bp, rbp, node.addr))
+                raise e
+            if t > 0 and s > 0:
+                print("%d bytes sent / %d bytes already up to date." % (t,s))
+            elif t > 0 and s == 0:
+                print("%d bytes sent." % (t))
             else:
-                print("Already up to date !")
+                print("Already up to date (%d bytes) !" % s)
 
         done.update(toSend)
 
@@ -311,8 +348,8 @@ class Testie:
 #                        fpath = './npf/' + fpath
                     fpath = os.path.relpath(fpath)
                     print("Sending files %s to %s... " % (fpath, role), end = '')
-                    t = node.executor.sendFolder(os.path.relpath(fpath,npf.npf_root()))
-                    if (t > 0):
+                    t = node.executor.sendFolder(fpath)
+                    if (t[0] > 0):
                         print("%d bytes sent." % t)
                     else:
                         print("Already up to date !")
@@ -322,7 +359,6 @@ class Testie:
 
     def test_tags(self):
         missings = []
-        # print("%s requires " % self.get_name(), self.config.get_list("require_tags"))
         for tag in self.config.get_list("require_tags"):
             if not tag in self.tags:
                 missings.append(tag)
@@ -359,14 +395,16 @@ class Testie:
                 print(p.strip())
             for node in npf.nodes_for_role(role):
                 if not node.executor.writeFile(filename, path_to_root, p):
-                    raise Exception("Could not create file %s on %s" % (filename, node.name))
+                    print("Re-trying with sudo...")
+                    if not node.executor.writeFile(filename, path_to_root, p, sudo=True):
+                        raise Exception("Could not create file %s on %s" % (filename, node.name))
 
     def test_require(self, v, build):
         for require in self.requirements + list(itertools.chain.from_iterable([imp.testie.requirements for imp in self.imports])):
             p = SectionVariable.replace_variables(v, require.content, require.role(),
                                                   self.config.get_dict("default_role_map"))
             pid, output, err, returncode = npf.executor(require.role(), self.config.get_dict("default_role_map")).exec(
-                cmd=p, bin_paths=[build.get_bin_folder()], options=self.options, event=None, testdir=None)
+                cmd=p, bin_paths=[build.get_local_bin_folder()], options=self.options, event=None, testdir=None)
             if returncode != 0:
                 return False, output, err
             continue
@@ -374,9 +412,12 @@ class Testie:
 
     def cleanup(self):
         for s in self.files:
+          try:
             path = Path(s.filename)
             if path.is_file():
                 path.unlink()
+          except:
+            print("Could not cleanup file %s" % s.filename)
 
     @staticmethod
     def killall(queue, event):
@@ -400,12 +441,30 @@ class Testie:
             except OSError:
                 pass
 
-    def update_constants(self, v_internals, build, full_test_folder, out_path):
-        v_internals.update({ 'NPF_REPO':get_valid_filename(build.repo.name),'NPF_ROOT': '../', 'NPF_BUILD': ('../' + build.build_path()) if not os.path.isabs(build.build_path()) else build.build_path(),
-                        'NPF_TESTIE_PATH':os.path.relpath(self.path if self.path else "./",full_test_folder),
-                        'NPF_RESULT_PATH':os.path.relpath(build.result_folder(), full_test_folder)})
+    def update_constants(self, v_internals : dict, build : Build, full_test_folder : str, out_path : str = None, node = None):
+
+        bp = ('../' + npf.get_build_path()) if not os.path.isabs(npf.get_build_path()) else npf.get_build_path()
+        rp = ('../' + build.build_path()) if not os.path.isabs(build.build_path()) else build.build_path()
+        abs_test_folder = full_test_folder if os.path.isabs(full_test_folder) else npf.cwd_path() + os.sep + full_test_folder
+
+        tp = os.path.relpath(self.path,abs_test_folder)
+
+        if node and node.executor.path:
+            bp = os.path.relpath(bp, npf.experiment_path() + '/testfolder/')
+            rp = os.path.relpath(rp, npf.experiment_path() + '/testfolder/')
+        v_internals.update({
+                        'NPF_REPO':get_valid_filename(build.repo.name),
+                        'NPF_REPO_PATH': rp,
+                        'NPF_ROOT': '../', #Deprecated
+                        'NPF_ROOT_PATH': os.path.relpath(npf.npf_root_path(), abs_test_folder),
+                        'NPF_BUILD_PATH': bp,
+                        'NPF_BUILD_ROOT': bp, #Deprecated
+                        'NPF_SCRIPT_PATH': tp,
+                        'NPF_TEST_PATH': tp, #Deprecatefd
+                        'NPF_TESTIE_PATH': tp, #Deprecatefd
+                        'NPF_RESULT_PATH': os.path.relpath(build.result_folder(), abs_test_folder)})
         if out_path:
-            v_internals.update({'NPF_OUTPUT_PATH':os.path.relpath(out_path, full_test_folder)})
+            v_internals.update({'NPF_OUTPUT_PATH': os.path.relpath(out_path, abs_test_folder)})
 
     def parse_results(self, regex_list: str, output: str, new_kind_results: dict, new_data_results: dict) -> Tuple[
         bool, bool]:
@@ -497,15 +556,17 @@ class Testie:
             if repo.version is not None:
                 v_internals[repo.reponame.upper() + '_VERSION'] = repo.version
         v.update(v_internals)
+
         if test_folder is None:
             test_folder = self.make_test_folder()
             f_mine = True
         else:
             f_mine = False
-        if not os.path.exists(npf.npf_root() + '/' + test_folder):
-            os.mkdir(npf.npf_root() + '/' + test_folder)
+
+        if not os.path.exists(npf.experiment_path() + '/' + test_folder):
+            os.mkdir(npf.experiment_path() + '/' + test_folder)
         save_path = os.getcwd()
-        os.chdir(npf.npf_root() + '/' + test_folder)
+        os.chdir(npf.experiment_path() + '/' + test_folder)
 
         # Build file list
         file_list = []
@@ -525,9 +586,12 @@ class Testie:
             imp.imp_v = {}
             for k, val in imp.testie.variables.statics().items():
                 imp.imp_v[k] = val.makeValues()[0]
+
             imp.imp_v.update(v)
+
             for late_variables in imp.testie.get_late_variables():
                 imp.imp_v.update(late_variables.execute(imp.imp_v, imp.testie))
+
             if SectionScript.TYPE_INIT in allowed_types:
                 file_list.extend(imp.testie.build_file_list(imp.imp_v, imp.get_role(), files=imp.testie.init_files))
             else:
@@ -589,13 +653,28 @@ class Testie:
                         param = RemoteParameters()
                         param.sudo = script.params.get("sudo", False)
 
-                        full_test_folder = (node.executor.path if node.executor.path else '..') + os.sep + test_folder + os.sep
-                        self.update_constants(v, build, full_test_folder, out_path=None)
+                        remote_test_folder = node.experiment_path() + os.sep + test_folder + os.sep
+                        self.update_constants(v, build, remote_test_folder, out_path=None, node=node)
                         v["NPF_MULTI"] = i_multi
                         v["NPF_MULTI_ID"] = i_multi
                         v["NPF_MULTI_MAX"] = node.multi if node.multi is not None else 1
                         v["NPF_ARRAY_ID"] = (i_node * v["NPF_MULTI_MAX"]) + i_multi
                         v["NPF_ARRAY_MAX"] = len(nodes) * v["NPF_MULTI_MAX"]
+
+                        #Checking if the script has a filter
+                        c = True # Should we continue?
+                        for ik, iv in script.params.items():
+                            if ik.startswith('ifeq-'):
+                                ik=ik[5:]
+                                if ik not in v:
+                                    print("WARNING: Filtering for %s for script %s but it is not in the variables" % (ik, param.title))
+                                if v[ik] != iv:
+                                    c = False
+                                    break
+
+                        if not c:
+                            continue
+
                         if node.mode == "netns" and i_multi > 0:
                             param.virt = "ip netns exec npfns%d" % i_multi
                             param.sudo = True
@@ -623,9 +702,10 @@ class Testie:
                         param.role_id = i_node
                         param.default_role_map = self.config.get_dict("default_role_map")
                         param.delay = script.delay()
-                        deps_bin_path = [repo.get_bin_folder() for repo in script.get_deps_repos(self.options) if
+
+                        deps_bin_path = [repo.get_remote_bin_folder(node) for repo in script.get_deps_repos(self.options) if
                                          not repo.reponame in self.options.ignore_deps]
-                        param.bin_paths = deps_bin_path + [build.get_bin_folder()]
+                        param.bin_paths = deps_bin_path + [build.get_remote_bin_folder(node)]
                         param.testdir = test_folder
                         param.event = event
                         param.script = script
@@ -779,6 +859,10 @@ class Testie:
                     vs.update(v)
                     try:
                         exec(self.pyexit.content, vs)
+                    except SystemExit as e:
+                        if e.code != 0:
+                            print("ERROR WHILE EXECUTING PYEXIT SCRIPT: returned code %d" % e.code)
+                        pass
                     except Exception as e:
                         print("ERROR WHILE EXECUTING PYEXIT SCRIPT:")
                         print(e)
@@ -872,11 +956,10 @@ class Testie:
                         print("Could not find expected result '%s' !" % result_type)
                         has_err = True
 
-                if len(data_results) + sum([len(r) for k, r in all_kind_results.items()]) == 0:
-                    print("Could not find results !")
+                  if len(data_results) + sum([len(r) for k, r in all_kind_results.items()]) == 0:
+                    print("Could not find any results ! Something probably went wrong, check the output :")
 
                     has_err = True
-                    continue
 
                 if has_err:
                     if not self.options.show_full:
@@ -969,8 +1052,8 @@ class Testie:
         init_done = False
         test_folder = self.make_test_folder()
 
-        #All the following paths must be relative to the NPF test folder (that is something like NPF's folder/testie1234567/)
-        full_test_folder = npf.from_root(test_folder) + os.sep
+        #All the following paths must be relative to the NPF experiment root folder (that is something like NPF's folder/testie1234567/)
+        full_test_folder = npf.from_experiment_path(test_folder) + os.sep
 
         dirname, basename, ext = npf.splitpath(options.output if options.output != 'graph' else options.graph_filename)
         out_path = dirname + os.sep
@@ -995,7 +1078,10 @@ class Testie:
 
         for runs_this_pass in total_runs:  # Number of results to ensure for this run
             n = 0
-            for root_variables in self.variables.expand(method=options.expand):
+            overriden = set(build.repo.overriden_variables.keys())
+            all_variables = list(self.variables.expand(method=options.expand, overriden=overriden))
+            n_tests = len(all_variables)
+            for root_variables in all_variables:
                 n += 1
 
                 variables = {}
@@ -1005,14 +1091,12 @@ class Testie:
                   if imp.is_include:
                     for k,v in imp.testie.variables.statics().items():
                         variables[k] = v.makeValues()[0]
-                  else:
-                    for k,v in imp.testie.variables.statics().items():
-                        shadow_variables[k] = v.makeValues()[0]
 
                 run = Run(variables)
                 variables.update(root_variables)
                 run.variables.update(build.repo.overriden_variables)
                 variables = run.variables.copy()
+
                 if shadow_variables:
                     shadow_variables.update(root_variables)
                     shadow_variables.update(build.repo.overriden_variables)
@@ -1022,8 +1106,10 @@ class Testie:
                     variables.update(late_variables.execute({**variables, **v_internals}, testie=self))
 
                 for imp in self.get_imports():
+                  if imp.is_include:
                     for late_variables in imp.testie.get_late_variables():
                         variables.update(late_variables.execute({**variables, **v_internals}, imp.testie))
+
                 r_status, r_out, r_err = self.test_require(variables, build)
                 if not r_status:
                     if not self.options.quiet:
@@ -1137,14 +1223,14 @@ class Testie:
                         if len(run_results) > 0:
                             if not dall:
                                 print("Results %s are missing some points..." % ", ".join(l))
-                        if len(self.variables) > 0:
+                        if n_tests > 0:
                             def print_header(i, i_try):
                                 n_try=int(self.config["n_retry"])
-                                print(run.format_variables(self.config["var_hide"]),
-                                  ("[%srun %d/%d for test %d/%d"+(" of serie %d/%d" %(iserie+1,nseries) if nseries > 1 else "")+"]") % (  ("retrying %d/%d " % (i_try + 1,n_try)) if i_try > 0 else "", i+1, n_runs, n, len(self.variables)))
-                        else:
-                            print("Executing single run...")
-
+                                desc = run.format_variables(self.config["var_hide"])
+                                if desc:
+                                    print(desc, end=' ')
+                                print(
+                                  ("[%srun %d/%d for test %d/%d"+(" of serie %d/%d" %(iserie+1,nseries) if nseries > 1 else "")+"]") % (  ("retrying %d/%d " % (i_try + 1,n_try)) if i_try > 0 else "", i+1, n_runs, n, n_tests))
 
                     new_data_results, new_all_kind_results, output, err, n_exec, n_err = self.execute(build, run, variables,
                                                                                                   n_runs,
@@ -1231,7 +1317,7 @@ class Testie:
 
         if not self.options.preserve_temp:
             try:
-                shutil.rmtree(npf.npf_root() + os.sep + test_folder)
+                shutil.rmtree(npf.experiment_path() + os.sep + test_folder)
             except PermissionError:
                 pass
         else:
@@ -1259,7 +1345,7 @@ class Testie:
     def expand_folder(testie_path, options, tags=None) -> List['Testie']:
         testies = []
         if not os.path.exists(testie_path):
-            print("The testie path %s does not exist" % testie_path)
+            print("The npf script path %s does not exist" % testie_path)
             return testies
         if os.path.isfile(testie_path):
             testie = Testie(testie_path, options=options, tags=tags)
@@ -1287,7 +1373,7 @@ class Testie:
                 else:
                     raise Exception(
                         "Roles %s cannot be on the same node ! Please use --cluster argument to set them accross nodes" % ' and '.join(
-                            s))
+                           testie.config.get_list("role_exclude")))
 
         return filtered_testies
 
@@ -1323,5 +1409,5 @@ class Testie:
 
     def make_test_folder(self):
         test_folder = "testie%s-%05d" % (datetime.datetime.now().strftime("%y%m%d%H%M"), random.randint(1, 2 << 16))
-        os.mkdir( npf.npf_root() + os.sep + test_folder)
+        os.mkdir(npf.experiment_path() + os.sep + test_folder)
         return test_folder
