@@ -7,6 +7,9 @@ from npf import npf
 from npf.nic import NIC
 from asteval import Interpreter
 import itertools
+import numpy as np
+import csv
+import sys
 
 import random
 
@@ -42,6 +45,8 @@ def get_bool(val):
 
 def is_bool(s):
     try:
+        if type(s) is list:
+            return False
         get_bool(s)
         return True
     except ValueError:
@@ -98,6 +103,16 @@ def is_log(l):
     return n
 
 
+def numeric_dict(d):
+        for k, v in d.items():
+            if type(v) is tuple:
+                if is_numeric(v[1]):
+                    d[k] = tuple(v[0],get_numeric(v[1]))
+            else:
+                if is_numeric(v):
+                    d[k] = get_numeric(v)
+        return d
+
 def ae_product_range(a,b):
     return itertools.product(range(a),range(b))
 
@@ -146,9 +161,8 @@ def replace_variables(v: dict, content: str, self_role=None, self_node=None, def
             else:
                 return str(v)
         else:
-            return str(nodes[nodeidx].get_nic(
-            int(nic_match.group('nic_idx') if nic_match.group('nic_idx') else v[nic_match.group('nic_var')]))[
-                       nic_match.group('type')])
+            nic = nodes[nodeidx].get_nic(int(nic_match.group('nic_idx')))
+            return str(nic[nic_match.group('type')])
 
     content = re.sub(
         Variable.VARIABLE_NICREF_REGEX,
@@ -174,12 +188,13 @@ def replace_variables(v: dict, content: str, self_role=None, self_node=None, def
     return content
 
 
+
 class VariableFactory:
     @staticmethod
     def build(name, valuedata, vsection=None):
-        result = re.match("\[(-?[0-9.]+)([+-]|[*]|[,])(-?[0-9.]+)([#][0-9.]+)?\]", valuedata)
+        result = re.match("(?P<doubleopen>\[?)\[(?P<a>-?[0-9.]+)(?P<log>[+-]|[*]|[,])(?P<b>-?[0-9.]+)(?P<step>[#][0-9.]*)?\](?P<doubleclose>\]?)", valuedata)
         if result:
-            return RangeVariable(name, result.group(1), result.group(3), result.group(2) == "*", (get_numeric(result.group(4)[1:]) if result.group(4) else None))
+            return RangeVariable(name, result.group('a'), result.group('b'), result.group('log') == "*", step= (get_numeric(result.group('step')[1:]) if result.group('step') else None), force_int=result.group('doubleopen')=='[')
 
         result = regex.match("\{([^:]*:[^,:]+)(?:(?:,)([^,:]*:[^,:]+))*\}", valuedata)
         if result:
@@ -188,9 +203,13 @@ class VariableFactory:
         result = regex.match("\{([^,]+)(?:(?:,)([^,]*))*}", valuedata)
         if result:
             return ListVariable(name, result.captures(1) + result.captures(2))
+        if valuedata.strip() == "{}":
+            return DictVariable(name, {})
 
         result = regex.match("EXPAND\((.*)\)", valuedata)
         if result:
+            if vsection is None:
+                raise Exception("RANDOM variable without vsection",vsection)
             return ExpandVariable(name, result.group(1), vsection)
 
         result = regex.match("RANDOM[ ]*\([ ]*([^,]+)[ ]*,[ ]*([^,]+)[ ]*\)", valuedata)
@@ -218,24 +237,83 @@ class VariableFactory:
 # raise Exception("Unkown variable type : " + valuedata)
 
 class Variable:
-    def __init__(self):
+    def __init__(self, name):
         self.assign = '='
         self.is_default = False
+        self.name = name
 
     NAME_REGEX = r'[a-zA-Z0-9._-]+'
     TAGS_REGEX = r'[a-zA-Z0-9._,|!-]+'
-    VALUE_REGEX = r'[a-zA-Z0-9._/,{}^$-]+'
+    VALUE_REGEX = r'[a-zA-Z0-9._/,{}^$-:]+'
     VARIABLE_REGEX = r'(?<!\\)[$](' \
                      r'[{](?P<varname_in>' + NAME_REGEX + ')[}]|' \
                      r'(?P<varname_sp>' + NAME_REGEX + ')(?=}|[^a-zA-Z0-9_]|$))'
     MATH_REGEX = r'(?P<prefix>\\)?[$][(][(](?P<expr>.*?)[)][)]'
     ALLOWED_NODE_VARS = 'path|user|addr|tags|nfs|arch|port'
-    NICREF_REGEX = r'(?P<role>[a-z0-9]+)[:](:?(?P<nic_idx>[0-9]+)[:](?P<type>' + NIC.TYPES + '+)|(?P<node>'+ALLOWED_NODE_VARS+'|ip|multi|mode|node))'
+    NICREF_REGEX = r'(?P<role>[a-z0-9]+)[:](:?(?P<nic_idx>[0-9]+)[:](?P<type>' + NIC.TYPES + '+)|(?P<node>'+ALLOWED_NODE_VARS+'|ip|ip6|multi|mode|node))'
     VARIABLE_NICREF_REGEX = r'(?<!\\)[$][{]' + NICREF_REGEX + '[}]'
 
+class ExperimentalDesign:
+    matrix = None
+    varmap = OrderedDict()
+
+    @classmethod
+    def getVals(cls, v:Variable):
+        if cls.matrix is None:
+            cls.load()
+        if v.name not in cls.varmap:
+            cls.varmap[v.name] = len(cls.varmap)
+        return cls.matrix[cls.varmap[v.name]]
+
+    @classmethod
+    def load(cls):
+        path = npf.find_local(sys.modules["npf.npf"].options.experimental_design)
+        assert path is not None
+
+        with open(path) as fd:
+            csvreader = csv.reader(fd)
+            data = [i for i in csvreader]
+        
+        ExperimentalDesign.matrix = np.array([[float(j) for j in i] for i in data])
+
+        # TODO: assert that the number of rows of the matrix is sufficient for all the values of the experimental design variables
+
+class CoVariable(Variable):
+    def __init__(self, name = "covariable"):
+        super().__init__(name)
+        self.vlist = OrderedDict()
+
+    def makeValues(self):
+        vs = [OrderedDict() for i in range(self.count())]
+        vals = []
+        for i,(var,val) in enumerate(self.vlist.items()):
+            vals.append(val.makeValues())
+        for i in range(self.count()):
+            for j,var in enumerate(self.vlist.keys()):
+                vs[i][var] = vals[j][i]
+        return vs
+
+    def count(self):
+        if len(self.vlist) == 0:
+            return 1
+        return min([v.count() for k,v in self.vlist.items()])
+
+    def format(self):
+        k = []
+        v = []
+        for var,val in self.vlist.items():
+            K,V = val.format()
+            k.append(K)
+            v.append(V)
+        return k,v
+
+    def is_numeric(self):
+        return False
+
 # For each value N of nums, generate a variable with the first N element of values
 class HeadVariable(Variable):
     def __init__(self, name, nums, values, join = None):
+        super().__init__(name)
         self.values = values
         if not is_numeric(nums):
             raise Exception("%s is not a number!" % nums)
@@ -257,7 +335,7 @@ class HeadVariable(Variable):
         return sum(self.nums) if len(self.nums) > 0 else 1
 
     def format(self):
-        return str
+        return self.name, str
 
     def is_numeric(self):
         return False
@@ -266,6 +344,7 @@ class ExpandVariable(Variable):
     """ Create a list wihich expands a string with all possible value for the variable
         it contains like it would be in a script or file section"""
     def __init__(self, name, value, vsection):
+        super().__init__(name)
         self.values = vsection.replace_all(value)
 
     def makeValues(self):
@@ -275,7 +354,7 @@ class ExpandVariable(Variable):
         return len(self.values)
 
     def format(self):
-        return str
+        return self.name, str
 
     def is_numeric(self):
         return False
@@ -290,7 +369,7 @@ class ExpandVariable(Variable):
 
 class SimpleVariable(Variable):
     def __init__(self, name, value):
-        super().__init__()
+        super().__init__(name)
         self.value = get_numeric(value)
 
     def makeValues(self):
@@ -300,7 +379,7 @@ class SimpleVariable(Variable):
         return 1
 
     def format(self):
-        return dtype(self.value)
+        return self.name, dtype(self.value)
 
     def is_numeric(self):
         return self.format() != str
@@ -318,7 +397,7 @@ class SimpleVariable(Variable):
 
 class ListVariable(Variable):
     def __init__(self, name, l):
-        super().__init__()
+        super().__init__(name)
         all_num = True
         for x in l:
             if not is_numeric(x):
@@ -350,10 +429,10 @@ class ListVariable(Variable):
         if t is int and is_bool(self.lvalues[0]):
             unique = list(set(self.lvalues))
             if len(unique) == 2 and all([int(u) in (0, 1) for u in unique]):
-                return bool
+                return self.name, bool
             elif len(unique) == 1 and (unique[0] == 'true' or unique[0] == 'false'):
-                return bool
-        return t
+                return self.name, bool
+        return self.name, t
 
     def is_numeric(self):
         return self.all_num
@@ -366,7 +445,7 @@ class ListVariable(Variable):
 
 class DictVariable(Variable):
     def __init__(self, name, data):
-        super().__init__()
+        super().__init__(name)
         if type(data) is dict:
             self.vdict = data
         else:
@@ -383,7 +462,7 @@ class DictVariable(Variable):
 
     def format(self):
         k, v = next(self.vdict.items().__iter__())
-        return dtype(v)
+        return self.name, dtype(v)
 
     def is_numeric(self):
         k, v = next(self.vdict.items().__iter__())
@@ -399,8 +478,9 @@ class DictVariable(Variable):
         return self
 
 class RangeVariable(Variable):
-    def __init__(self, name, valuestart, valueend, log, step = None):
-        super().__init__()
+    def __init__(self, name, valuestart, valueend, log, step = None, force_int = False):
+        super().__init__(name)
+
         if is_integer(valuestart) and is_integer(valueend):
             valuestart=int(valuestart)
             valueend=int(valueend)
@@ -421,44 +501,53 @@ class RangeVariable(Variable):
                 self.step = 1
         else:
             self.step = step
+        self.force_int = force_int
 
     def count(self):
         """todo: think"""
-        if self.log:
-            return len(self.makeValues())
+        if self.step == "":
+            return len(ExperimentalDesign.getVals(self))
         else:
-            return int(((self.b-self.a) / self.step) + 1)
+            if self.log:
+                return len(self.makeValues())
+            else:
+                return int(((self.b-self.a) / self.step) + 1)
 
 
     def makeValues(self):
-            vs = []
-            i = self.a
-            while i <= self.b:
-                vs.append(i)
-                if i == self.b:
-                    break
-                if i == 0 and self.log:
-                    if self.b > 0:
-                        i = 1
+            #Experimental design
+            if self.step == "":
+                vs =  self.a + (self.b-self.a) * ExperimentalDesign.getVals(self)
+            else:
+                vs = []
+                i = self.a
+                while i <= self.b:
+                    vs.append(i)
+                    if i == self.b:
+                        break
+                    if i == 0 and self.log:
+                        if self.b > 0:
+                            i = 1
+                        else:
+                            i = -1
                     else:
-                        i = -1
-                else:
-                    if self.log:
-                        i *= self.step
-                    else:
-                        i += self.step
-            if i > self.b:
-                vs.append(self.b)
-            return vs
+                        if self.log:
+                            i *= self.step
+                        else:
+                            i += self.step
+                if i > self.b:
+                    vs.append(self.b)
+            return [int(v) for v in vs] if self.force_int else vs
 
     def format(self):
-        return int
+        return self.name, int
 
     def is_numeric(self):
         return True
 
 class IfVariable(Variable):
     def __init__(self, name, cond, a, b):
+        super().__init__(name)
         self.cond = cond
         self.a = a
         self.b = b
@@ -481,6 +570,7 @@ class IfVariable(Variable):
 
 class RandomVariable(Variable):
     def __init__(self, name, a, b):
+        super().__init__(name)
         self.a = int(a.strip())
         self.b = int(b.strip())
 
@@ -492,7 +582,7 @@ class RandomVariable(Variable):
         return 1
 
     def format(self):
-        return int
+        return self.name, int
 
     def is_numeric(self):
         return True
